#!/bin/bash

PRE_ETH=ueth

update_if_name(){
    ip link set "$1" name "$2"
    grep -rnwl "$1" /etc/network/ | xargs sed -i "s/\b$1\b/$2/g"
    mv /etc/network/interfaces.d/"$1" /etc/network/interfaces.d/"$2"
    mv /etc/network/interfaces.d/"$1.d" /etc/network/interfaces.d/"$2.d"
}

lock_if_name(){
    if_no=1
    for if_path in /sys/class/net/*;do
        if_name=`basename $if_path`
        if [ ! -e "/sys/devices/virtual/net/$if_name" ];then
            # Physical interface

            # Skip USB interface
            realpath $if_path | grep -q '/usb[0-9]/' && continue

            iname="$PRE_ETH$if_no"

            write_link "$iname" `cat $if_path/address`
            if ifquery --state "$if_name";then
                if ovs-vsctl iface-to-br "$if_name";then
                    # In ovs
                    ovs_br=`ovs-vsctl iface-to-br "$if_name"`
                    if ovs-vsctl list-ports "$ovs_br" | grep -w "$if_name";then
                        # Normal port
                        ovs-vsctl del-port "$if_name"
                        ovs-vsctl add-port "$ovs_br" "$iname"
                    else
                        # In bond
                        # 运行 ovs-vsctl show 并将输出存储到变量 output 中
                        output=$(ovs-vsctl show)

                        # 使用 awk 解析 output，查找 if_name 所在的 bond
                        bridge=""
                        bond=""

                        while read -r line; do
                            if [[ $line =~ ^[[:space:]]*Port[[:space:]] ]]; then
                                bond=$(echo "$line" | awk '{print $2}')
                            elif [[ $line =~ ^[[:space:]]*Interface[[:space:]]$if_name ]]; then
                                break
                            fi
                        done <<< "$output"
                        ovs-vsctl del-bond-iface "$if_name"
                        ovs-vsctl add-bond-iface "$bond" "$iname"
                    fi
                fi

                ifdown "$if_name"
                update_if_name "$if_name" "$iname"
                systemctl start ifup-raw@$iname.service

            else
                update_if_name "$if_name" "$iname"
            fi

            ((if_no++)); :
        fi
    done
}

write_link(){
cat > /lib/systemd/network/10-$1.link <<- LINK
[Match]
MACAddress=$2

[Link]
Name=$1
LINK
}

if [ ! -e /lib/systemd/network/10-${PRE_ETH}1.link ];then
    lock_if_name
fi
